@@ -180,6 +180,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_tree("false() or true()", '(or (false) (true))')
         self.check_tree("./A/B[C][D]/E", '(/ (/ (/ (.) (A)) ([ ([ (B) (C)) (D))) (E))')
         self.check_tree("string(xml:lang)", '(string (: (xml) (lang)))')
+        self.check_tree("//text/preceding-sibling::text[1]",
+                        '(/ (// (text)) ([ (preceding-sibling (text)) (1)))')
 
     def test_token_source(self):
         self.check_source(' child ::B1', 'child::B1')
@@ -357,11 +359,11 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         root = self.etree.XML('<A><B1><C1/><C2/></B1><B2/><B3><C3/><C4/><C5/></B3></A>')
         context = XPathContext(root, item=root[1], size=3, position=3)
         self.check_value("position()", 0)
-        self.check_value("position()", 4, context=context)
+        self.check_value("position()", 3, context=context)
         self.check_value("position()<=2", True)
         self.check_value("position()<=2", False, context=context)
-        self.check_value("position()=4", True, context=context)
-        self.check_value("position()=3", False, context=context)
+        self.check_value("position()=3", True, context=context)
+        self.check_value("position()=2", False, context=context)
         self.check_value("last()", 0)
         self.check_value("last()", 3, context=context)
         self.check_value("last()-1", 2, context=context)
@@ -428,13 +430,13 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         if self.parser.version == '1.0':
             self.wrong_syntax('fn:normalize-space(())')
             self.check_value("normalize-space(1000)", '1000')
-            self.check_value("normalize-space(true())", 'True')
+            self.check_value("normalize-space(true())", 'true')
         else:
             self.check_value('fn:normalize-space(())', '')
             self.wrong_type("normalize-space(true())")
             self.wrong_type("normalize-space(('\ta  b c  ', 'other'))")
             self.parser.compatibility_mode = True
-            self.check_value("normalize-space(true())", 'True')
+            self.check_value("normalize-space(true())", 'true')
             self.check_value("normalize-space(('\ta  b\tc  ', 'other'))", 'a b c')
             self.parser.compatibility_mode = False
 
@@ -638,6 +640,9 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("substring-after('Wolfgang Amadeus Mozart', 'Amadeus ')", 'Mozart')
         self.check_value("substring-after('Wolfgang Amadeus Mozart', 'Mozart')", '')
         self.check_value("substring-after('', '')", '')
+        self.check_value("substring-after('Mozart', 'B')", '')
+        self.check_value("substring-after('Mozart', 'Bach')", '')
+        self.check_value("substring-after('Mozart', 'Amadeus')", '')
         self.check_value("substring-after('Mozart', '')", 'Mozart')
         self.check_value('substring-after("1999/04/01","/")', '04/01')
         self.check_value('substring-after("1999/04/01","19")', '99/04/01')
@@ -679,6 +684,32 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         else:
             self.check_value("boolean(())", False)
 
+    def test_boolean_context_nonempty_elements(self):
+        root = self.etree.XML("""<a> <b>text</b> </a>""")
+        context = XPathContext(root=root)
+
+        root_token = self.parser.parse("boolean(node())")
+        self.assertEqual(True, root_token.evaluate(context))
+
+        root_token = self.parser.parse("not(node())")
+        self.assertEqual(False, root_token.evaluate(context))
+        root_token = self.parser.parse("not(not(node()))")
+        self.assertEqual(True, root_token.evaluate(context))
+
+    def test_nonempty_elements(self):
+        root = self.etree.XML("""<a> <b>text</b> </a>""")
+        context = XPathContext(root=root)
+
+        root_token = self.parser.parse("normalize-space(text()) = ''")
+        self.assertEqual(True, root_token.evaluate(context))
+
+        elements = select(root, "//*")
+        for element in elements:
+            context = XPathContext(root=root, item=element)
+
+            root_token = self.parser.parse("* or normalize-space(text()) != ''")
+            self.assertEqual(True, root_token.evaluate(context), element)
+
     def test_lang_function(self):
         # From https://www.w3.org/TR/1999/REC-xpath-19991116/#section-Boolean-Functions
         self.check_selector('lang("en")', self.etree.XML('<para xml:lang="en"/>'), True)
@@ -687,10 +718,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_selector('lang("en")', self.etree.XML('<para xml:lang="en-us"/>'), True)
         self.check_selector('lang("en")', self.etree.XML('<para xml:lang="it"/>'), False)
 
-    def test_logical_expressions(self):
+    def test_logical_and_operator(self):
         self.check_value("false() and true()", False)
-        self.check_value("false() or true()", True)
-        self.check_value("true() or false()", True)
         self.check_value("true() and true()", True)
         self.check_value("1 and 0", False)
         self.check_value("1 and 1", True)
@@ -698,6 +727,25 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("0 and 'mars'", False)
         self.check_value("1 and mars", False)
 
+    def test_logical_or_operator(self):
+        self.check_value("false() or true()", True)
+        self.check_value("true() or false()", True)
+
+    def test_logical_expressions(self):
+        root_token = self.parser.parse("(@a and not(@b)) or (not(@a) and @b)")
+        context = XPathContext(self.etree.XML('<root a="10" b="0"/>'))
+        self.assertTrue(root_token.evaluate(context=context) is False)
+        context = XPathContext(self.etree.XML('<root a="10" b="1"/>'))
+        self.assertTrue(root_token.evaluate(context=context) is False)
+        context = XPathContext(self.etree.XML('<root a="10"/>'))
+        self.assertTrue(root_token.evaluate(context=context) is True)
+        context = XPathContext(self.etree.XML('<root a="0"/>'))
+        self.assertTrue(root_token.evaluate(context=context) is True)
+        context = XPathContext(self.etree.XML('<root b="0"/>'))
+        self.assertTrue(root_token.evaluate(context=context) is True)
+        context = XPathContext(self.etree.XML('<root/>'))
+        self.assertTrue(root_token.evaluate(context=context) is False)
+
     def test_comparison_operators(self):
         self.check_value("0.05 = 0.05", True)
         self.check_value("19.03 != 19.02999", True)
@@ -735,6 +783,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         self.check_value("9 - 1 + 6", 14)
         self.check_value("(5 * 7) + 9", 44)
         self.check_value("-3 * 7", -21)
+        self.check_value('(2 + 4) * 5', 30)
+        self.check_value('2 + 4 * 5', 22)
 
     def test_div_operator(self):
         self.check_value("5 div 2", 2.5)
@@ -1216,6 +1266,45 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             # XPath 1.0 ignores the default namespace declarations
             self.check_selector("name(B1)", root, '', namespaces={'': "http://xpath.test/ns"})
 
+    def test_issue_25_with_count_function(self):
+        root = lxml_etree.fromstring("""
+            <line>
+                <text size="12.482">C</text>
+                <text size="12.333">A</text>
+                <text size="12.333">P</text>
+                <text size="12.333">I</text>
+                <text size="12.482">T</text>
+                <text size="12.482">O</text>
+                <text size="12.482">L</text>
+                <text size="12.482">O</text>
+                <text></text>
+                <text size="12.482">I</text>
+                <text size="12.482">I</text>
+                <text size="12.482">I</text>
+                <text></text>
+            </line>""")
+
+        path = '//text/preceding-sibling::text'
+        self.check_selector(path, root, root[:-1])
+
+        self.check_tree('//text[7]/preceding-sibling::text[1]',
+                        '(/ (// ([ (text) (7))) ([ (preceding-sibling (text)) (1)))')
+
+        if self.parser.version != '1.0':
+            self.check_tree('//text[7]/(preceding-sibling::text)[1]',
+                            '(/ (// ([ (text) (7))) ([ (preceding-sibling (text)) (1)))')
+            path = '//text[7]/(preceding-sibling::text)[2]'
+            self.check_selector(path, root, [root[1]])
+
+        path = '//text[7]/preceding-sibling::text[2]'
+        self.check_selector(path, root, [root[4]])
+
+        path = 'count(//text[@size="12.482"][not(preceding-sibling::text[1][@size="12.482"])])'
+        self.check_selector(path, root, 3)
+
+        path = '//text[@size="12.482"][not(preceding-sibling::text[1][@size="12.482"])]'
+        self.check_selector(path, root, [root[0], root[4], root[9]])
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath1ParserTest(XPath1ParserTest):
