@@ -18,7 +18,7 @@ try:
 except ImportError:
     lxml_etree = None
 
-from elementpath import XPathContext, AttributeNode
+from elementpath import XPathContext, AttributeNode, TypedAttribute
 from elementpath.datatypes import Timezone, DateTime, GregorianYear10, \
     Duration, YearMonthDuration, DayTimeDuration, Date10, Time, \
     XPathGregorianDay, XPathGregorianMonth, XPathGregorianMonthDay, \
@@ -282,11 +282,14 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         context = XPathContext(root)
         self.check_value('xs:dateTime(@a)', DateTime(1969, 7, 20, 20, 18), context=context)
 
-        context.item = AttributeNode('a', DateTime(1969, 7, 20, 20, 18))
+        context.item = AttributeNode('a', str(DateTime(1969, 7, 20, 20, 18)))
         self.check_value('xs:dateTime(.)', DateTime(1969, 7, 20, 20, 18), context=context)
 
-        context.item = AttributeNode('a', True)
-        self.check_value('xs:dateTime(.)', TypeError, context=context)
+        context.item = AttributeNode('a', 'true')
+        self.check_value('xs:dateTime(.)', ValueError, context=context)
+
+        context.item = DateTime(1969, 7, 20, 20, 18)
+        self.check_value('xs:dateTime(.)', DateTime(1969, 7, 20, 20, 18), context=context)
 
     def test_time_constructor(self):
         tz0 = None
@@ -323,12 +326,27 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         context = XPathContext(root)
         self.check_value('xs:date(@a)', Date10(2017, 1, 19), context=context)
 
-        context.item = AttributeNode('a', Date10(2017, 1, 19))
+        context.item = AttributeNode('a', str(Date10(2017, 1, 19)))
         self.check_value('xs:date(.)', Date10(2017, 1, 19), context=context)
 
+        context.item = AttributeNode('a', 'true')
+        self.check_value('xs:date(.)', ValueError, context=context)
         context.item = AttributeNode('a', True)
+        self.check_value('xs:date(.)', ValueError, context=context)
+        context.item = TypedAttribute(AttributeNode('a', 'true'), True)
         self.check_value('xs:date(.)', TypeError, context=context)
 
+        root = self.etree.XML("<root>2017-10-02</root>")
+        context = XPathContext(root)
+        self.check_value('xs:date(.)', Date10(2017, 10, 2), context=context)
+
+        root = self.etree.XML("<root>2017<a>-10-02</a></root>")
+        context = XPathContext(root)
+        self.check_value('xs:date(.)', Date10(2017, 10, 2), context=context)
+
+        context = XPathContext(root, item=Date10(2017, 10, 2))
+        self.check_value('xs:date(.)', Date10(2017, 10, 2), context=context)
+
     def test_gregorian_day_constructor(self):
         tz0 = None
         tz1 = Timezone(datetime.timedelta(hours=5, minutes=24))
@@ -432,6 +450,10 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         context.item = Duration(months=12, seconds=86400)
         self.check_value('xs:duration(.)', Duration(12, 86400), context=context)
 
+        root = self.etree.XML('<root><a>P1Y5M</a></root>')
+        context = XPathContext(root)
+        self.check_value('xs:duration(.)', Duration(months=17), context=context)
+
     def test_year_month_duration_constructor(self):
         self.check_value('xs:yearMonthDuration("P3Y5M")', (41, 0))
         self.check_value('xs:yearMonthDuration(xs:untypedAtomic("P3Y5M"))', (41, 0))
@@ -492,10 +514,8 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
 
         context.item = b'ODQ='
         if platform.python_implementation() != 'PyPy':
+            # Skip for PyPy because PyPy 7.1.1-beta0 has a bug on codecs.decode()
             self.check_value('xs:hexBinary(.)', b'3834', context=context)
-        else:
-            # PyPy 7.1.1-beta0 bug on codecs.decode() implementation
-            self.check_value('xs:hexBinary(.)', TypeError, context=context)
 
     def test_base64_binary_constructor(self):
         self.check_value('xs:base64Binary(())', [])
@@ -514,10 +534,8 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
 
         context.item = b'FF'
         if platform.python_implementation() != 'PyPy':
+            # Skip for PyPy because PyPy 7.1.1-beta0 has a bug on codecs.decode()
             self.check_value('xs:base64Binary(.)', b'/w==\n', context=context)
-        else:
-            # PyPy 7.1.1-beta0 bug on codecs.decode() implementation
-            self.check_value('xs:base64Binary(.)', TypeError, context=context)
 
         context.item = b'abcefghi'  # Don't change, it can be an encoded value.
         self.check_value('xs:base64Binary(.)', b'abcefghi', context=context)
@@ -525,6 +543,16 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         context.item = b'abcefghij'
         self.check_value('xs:base64Binary(.)', b'YWJjZWZnaGlq\n', context=context)
 
+    def test_untyped_atomic_constructor(self):
+        self.check_value('xs:untypedAtomic(())', [])
+
+        root = self.etree.XML('<root>1999</root>')
+        context = XPathContext(root)
+        self.check_value('xs:untypedAtomic(.)', UntypedAtomic(1999), context=context)
+
+        context.item = UntypedAtomic('true')
+        self.check_value('xs:untypedAtomic(.)', UntypedAtomic(True), context=context)
+
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath2ConstructorsTest(XPath2ConstructorsTest):
