@@ -154,6 +154,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
     def test_max_function(self):
         self.check_value("fn:max((3,4,5))", 5)
+        self.check_value("fn:max((3, 4, xs:float('NaN')))", float('nan'))
         self.check_value("fn:max((3,4,5), 'en_US.UTF-8')", 5)
         self.check_value("fn:max((5, 5.0e0))", 5.0e0)
         self.wrong_type("fn:max((3,4,'Zero'))")
@@ -167,6 +168,7 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
 
     def test_min_function(self):
         self.check_value("fn:min((3,4,5))", 3)
+        self.check_value("fn:min((3, 4, xs:float('NaN')))", float('nan'))
         self.check_value("fn:min((5, 5.0e0))", 5.0e0)
         self.check_value("fn:min((xs:float(0.0E0), xs:float(-0.0E0)))", 0.0)
         self.check_value('fn:min((fn:current-date(), xs:date("2001-01-01")))',
@@ -197,67 +199,72 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         env_locale_setting = locale.getlocale(locale.LC_COLLATE)
 
         locale.setlocale(locale.LC_COLLATE, 'C')
-        self.assertEqual(locale.getlocale(locale.LC_COLLATE), (None, None))
-
-        self.check_value("fn:compare('abc', 'abc')", 0)
-        self.check_value("fn:compare('abc', 'abd')", -1)
-        self.check_value("fn:compare('abc', 'abb')", 1)
-        self.check_value("fn:compare('foo bar', 'foo bar')", 0)
-        self.check_value("fn:compare('', '')", 0)
-        self.check_value("fn:compare('abc', 'abcd')", -1)
-        self.check_value("fn:compare('', ' foo bar')", -1)
-        self.check_value("fn:compare('abcd', 'abc')", 1)
-        self.check_value("fn:compare('foo bar', '')", 1)
-
-        self.check_value('fn:compare("a","A")', 1)
-        self.check_value('fn:compare("A","a")', -1)
-        self.check_value('fn:compare("+++","++")', 1)
-        self.check_value('fn:compare("1234","123")', 1)
-
-        self.check_value("fn:count(fn:compare((), ''))", 0)
-        self.check_value("fn:count(fn:compare('abc', ()))", 0)
-
-        self.check_value("compare(xs:anyURI('http://example.com/'), 'http://example.com/')", 0)
-        self.check_value(
-            "compare(xs:untypedAtomic('http://example.com/'), 'http://example.com/')", 0
-        )
-
-        self.check_value('compare("&#65537;", "&#65538;", '
-                         '"http://www.w3.org/2005/xpath-functions/collation/codepoint")', -1)
-
-        self.check_value('compare("&#65537;", "&#65520;", '
-                         '"http://www.w3.org/2005/xpath-functions/collation/codepoint")', 1)
-
-        # Issue #17
-        self.check_value("fn:compare('Strassen', 'Straße')", -1)
-
-        locale.setlocale(locale.LC_COLLATE, 'en_US.UTF-8')
-        self.check_value("fn:compare('Strasse', 'Straße')", -1)
-        self.check_value("fn:compare('Strassen', 'Straße')", 1)
-
         try:
-            self.check_value("fn:compare('Strasse', 'Straße', 'it_IT.UTF-8')", -1)
+            self.assertEqual(locale.getlocale(locale.LC_COLLATE), (None, None))
+
+            self.check_value("fn:compare('abc', 'abc')", 0)
+            self.check_value("fn:compare('abc', 'abd')", -1)
+            self.check_value("fn:compare('abc', 'abb')", 1)
+            self.check_value("fn:compare('foo bar', 'foo bar')", 0)
+            self.check_value("fn:compare('', '')", 0)
+            self.check_value("fn:compare('abc', 'abcd')", -1)
+            self.check_value("fn:compare('', ' foo bar')", -1)
+            self.check_value("fn:compare('abcd', 'abc')", 1)
+            self.check_value("fn:compare('foo bar', '')", 1)
+
+            self.check_value('fn:compare("a","A")', 1)
+            self.check_value('fn:compare("A","a")', -1)
+            self.check_value('fn:compare("+++","++")', 1)
+            self.check_value('fn:compare("1234","123")', 1)
+
+            self.check_value("fn:count(fn:compare((), ''))", 0)
+            self.check_value("fn:count(fn:compare('abc', ()))", 0)
+
+            self.check_value("compare(xs:anyURI('http://example.com/'), 'http://example.com/')", 0)
+            self.check_value(
+                "compare(xs:untypedAtomic('http://example.com/'), 'http://example.com/')", 0
+            )
+
+            self.check_value('compare("&#65537;", "&#65538;", '
+                             '"http://www.w3.org/2005/xpath-functions/collation/codepoint")', -1)
+
+            self.check_value('compare("&#65537;", "&#65520;", '
+                             '"http://www.w3.org/2005/xpath-functions/collation/codepoint")', 1)
+
+            # Issue #17
+            self.check_value("fn:compare('Strassen', 'Straße')", -1)
+
+            if platform.system() != 'Linux':
+                return
+
+            locale.setlocale(locale.LC_COLLATE, 'en_US.UTF-8')
+            self.check_value("fn:compare('Strasse', 'Straße')", -1)
             self.check_value("fn:compare('Strassen', 'Straße')", 1)
-        except locale.Error:
-            pass  # Skip test if 'it_IT.UTF-8' is an unknown locale setting
 
-        try:
-            self.check_value("fn:compare('Strasse', 'Straße', 'de_DE.UTF-8')", -1)
-        except locale.Error:
-            pass  # Skip test if 'de_DE.UTF-8' is an unknown locale setting
+            try:
+                self.check_value("fn:compare('Strasse', 'Straße', 'it_IT.UTF-8')", -1)
+                self.check_value("fn:compare('Strassen', 'Straße')", 1)
+            except locale.Error:
+                pass  # Skip test if 'it_IT.UTF-8' is an unknown locale setting
 
-        try:
-            self.check_value("fn:compare('Strasse', 'Straße', 'deutsch')", -1)
-        except locale.Error:
-            pass  # Skip test if 'deutsch' is an unknown locale setting
+            try:
+                self.check_value("fn:compare('Strasse', 'Straße', 'de_DE.UTF-8')", -1)
+            except locale.Error:
+                pass  # Skip test if 'de_DE.UTF-8' is an unknown locale setting
 
-        with self.assertRaises(locale.Error) as cm:
-            self.check_value("fn:compare('Strasse', 'Straße', 'invalid_collation')")
-        self.assertIn('FOCH0002', str(cm.exception))
+            try:
+                self.check_value("fn:compare('Strasse', 'Straße', 'deutsch')", -1)
+            except locale.Error:
+                pass  # Skip test if 'deutsch' is an unknown locale setting
 
-        self.wrong_type("fn:compare('Strasse', 111)")
+            with self.assertRaises(locale.Error) as cm:
+                self.check_value("fn:compare('Strasse', 'Straße', 'invalid_collation')")
+            self.assertIn('FOCH0002', str(cm.exception))
 
-        locale.setlocale(locale.LC_COLLATE, env_locale_setting)
+            self.wrong_type("fn:compare('Strasse', 111)")
+
+        finally:
+            locale.setlocale(locale.LC_COLLATE, env_locale_setting)
 
     def test_normalize_unicode_function(self):
         self.check_value('fn:normalize-unicode(())', '')
@@ -517,11 +524,13 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
         self.check_value('fn:empty(fn:remove(("hello", "world"), 1))', False)
         self.check_value('fn:empty(())', True)
         self.check_value('fn:empty(fn:remove(("hello"), 1))', True)
+        self.check_value('fn:empty((xs:double("0")))', False)
 
     def test_exists_function(self):
         self.check_value('fn:exists(("hello", "world"))', True)
         self.check_value('fn:exists(())', False)
         self.check_value('fn:exists(fn:remove(("hello"), 1))', False)
+        self.check_value('fn:exists((xs:int("-1873914410")))', True)
 
     def test_distinct_values_function(self):
         self.check_value('fn:distinct-values((1, 2.0, 3, 2))', [1, 2.0, 3])
@@ -667,12 +676,9 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                   <xs:element name="root"/>
                 </xs:schema>""")
 
-            self.parser.schema = xmlschema.xpath.XMLSchemaProxy(schema)
-            context = self.parser.schema.get_context()
-            try:
+            with self.schema_bound_parser(schema.xpath_proxy):
+                context = self.parser.schema.get_context()
                 self.check_value("fn:resolve-QName('p3:C3', .)", None, context)
-            finally:
-                self.parser.schema = None
 
     def test_namespace_uri_for_prefix_function(self):
 
@@ -728,13 +734,10 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                   <xs:element name="root"/>
                 </xs:schema>""")
 
-            self.parser.schema = xmlschema.xpath.XMLSchemaProxy(schema)
-            context = self.parser.schema.get_context()
-            prefixes = {'xml', 'xs', 'xlink', 'fn', 'err', 'xsi', 'eg', 'tst'}
-            try:
+            with self.schema_bound_parser(schema.xpath_proxy):
+                context = self.parser.schema.get_context()
+                prefixes = {'xml', 'xs', 'xlink', 'fn', 'err', 'xsi', 'eg', 'tst'}
                 self.check_value("fn:in-scope-prefixes(.)", prefixes, context)
-            finally:
-                self.parser.schema = None
 
     def test_datetime_function(self):
         tz0 = None
@@ -1067,7 +1070,9 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                          Time.fromstring('03:00:00+10:00'), context)
 
     def test_default_collation_function(self):
-        self.check_value('fn:default-collation()', 'en_US.UTF-8')
+        default_locale = locale.getdefaultlocale()
+        default_locale = '.'.join(default_locale) if default_locale[1] else default_locale[0]
+        self.check_value('fn:default-collation()', default_locale)
 
     def test_context_functions(self):
         context = XPathContext(root=self.etree.XML('<A/>'))
@@ -1192,12 +1197,9 @@ class XPath2FunctionsTest(xpath_test_class.XPathTestCase):
                   <xs:element name="root"/>
                 </xs:schema>""")
 
-            self.parser.schema = xmlschema.xpath.XMLSchemaProxy(schema)
-            context = self.parser.schema.get_context()
-            try:
+            with self.schema_bound_parser(schema.xpath_proxy):
+                context = self.parser.schema.get_context()
                 self.check_value("fn:root()", None, context)
-            finally:
-                self.parser.schema = None
 
     def test_error_function(self):
         with self.assertRaises(ElementPathError) as err:
